; crt0.s
; startup code for Ittiara

.PC02

.export	_init, _exit
.import	_main, _duart_init, DUART_OUTA

.export	__STARTUP__ : absolute = 1	; Mark as startup
.import	__RAM_START__, __RAM_SIZE__	; Generated by the linker

.import	initlib, donelib  ;, copydata, zerobss

.include "zeropage.inc"

; Startup code goes in its own segment
.segment "STARTUP"

;Basic 6502 housekeeping
_init:	CLI
		CLD
		LDX		#$FF
		TXS
		
;Set cc65 arg stack pointer
		LDA		#<(__RAM_START__ + __RAM_SIZE__)
		STA		sp
		LDA		#>(__RAM_START__ + __RAM_SIZE__)
		STA		sp+1

;Initialize memory storage
		LDA #$FF
		STA  $7FE2
		
		;JSR _duart_init
		
		TSX
		TXA
		;JSR DUART_OUTA
		STA  $7FE0
		
		JSR		zerobss		;Clear the BSS segment
		
		;TSX
		;STX  $7FE0
		;WAI
		
		JSR		copydata	;Initialize the DATA segment
		
		JSR		initlib		;Run constructors
		
;Call main()
		JSR		_main

;On return from main() or as a call to exit(), force a software interrupt.
_exit:	JSR		donelib		;Run destructors
		BRK
		
;Include some of the C runtime here, so that it can be instrumented.

;zerobss
        .import         __BSS_RUN__, __BSS_SIZE__
        .importzp       ptr1


.code

zerobss:
		LDX #%11100000
		STX  $7FEC
		TSX
		DEX
		DEX
		LDA  $0100,X
		STA  $7FE0
		STP
		
        lda     #<__BSS_RUN__
        sta     ptr1
        lda     #>__BSS_RUN__
        sta     ptr1+1
        
        lda     #0
        tay

; Clear full pages

L1:		ldx     #>__BSS_SIZE__
        beq     L3
L2:     sta     (ptr1),y
        iny
        bne     L2
        inc     ptr1+1
        dex
        bne     L2

; Clear remaining page (y is zero on entry)

L3:     cpy     #<__BSS_SIZE__
        beq     L4
        sta     (ptr1),y
        iny
        bne     L3

; Done

L4:     rts

;copydata
        .import         __DATA_LOAD__, __DATA_RUN__, __DATA_SIZE__
        .importzp       ptr1, ptr2, tmp1


copydata:
        lda     #<__DATA_LOAD__         ; Source pointer
        sta     ptr1
        lda     #>__DATA_LOAD__
        sta     ptr1+1

        lda     #<__DATA_RUN__          ; Target pointer
        sta     ptr2
        lda     #>__DATA_RUN__
        sta     ptr2+1

        ldx     #<~__DATA_SIZE__
        lda     #>~__DATA_SIZE__        ; Use -(__DATASIZE__+1)
        sta     tmp1
        ldy     #$00

; Copy loop

@L1:    inx
        beq     @L3

@L2:    lda     (ptr1),y
        sta     (ptr2),y
        iny
        bne     @L1
        inc     ptr1+1
        inc     ptr2+1                  ; Bump pointers
        bne     @L1                     ; Branch always (hopefully)

; Bump the high counter byte

@L3:    inc     tmp1
        bne     @L2

; Done

        rts

