;duart.s - NXP 28L92 driver

;exports
.export DUART_INIT, DUART_INA, DUART_OUTA
.export _duart_a_available, _duart_ina_nw, _duart_ina, _duart_outa_nw, _duart_outa, _duart_init

.segment "CODE"

; register definitions
.define MODE_A		$0		;Mode Register A
.define STATUS_A	$1		;Status Register A
.define CLOCK_A		$1		;Clock Select Register A
.define COMMAND_A	$2		;Command Register A
.define FIFO_A		$3		;Rx/Tx FIFO A
.define IPCR		$4		;Input Port Configuration Register
.define ACR			$4		;Auxiliary Configuration Register
.define ISR			$5		;Interrupt Status Register
.define IMR			$5		;Interrupt Mask Register
.define CTU			$6		;Counter/Timer Upper byte
.define CTPU		$6		;Counter/Timer Preset Upper byte
.define CTL			$7		;Counter/Timer Lower byte
.define CTPL		$7		;Counter/Timer Preset Lower byte
.define MODE_B		$8		;Mode Register B
.define STATUS_B	$9		;Status Register B
.define CLOCK_B		$9		;Clock Select Register B
.define COMMAND_B	$A		;Command Register B
.define FIFO_B		$B		;Rx/Tx FIFO B
.define MISC		$C		;Miscellaneous register(a general-purpose memory location in Intel mode)
.define IPR			$D		;Input Port Register
.define OPCR		$D		;Output Port Configuration Register
.define SOPR		$E		;Set Output Port Register
.define ROPR		$F		;Reset Output Port Register

.define DUART		$7FD0	;base address of the DUART

DUART_INIT:
_duart_init:	;Set up both channels of the DUART for 115200 8N1, with no hardware handshaking.
		STZ  DUART+IMR		;Disable IRQs
		LDA #$BA
		STA  DUART+COMMAND_A		;Disable Tx & Rx; MRA -> MR0
		STA  DUART+COMMAND_B
		LDA #$66
		STA  DUART+CLOCK_A		; - Select 115,200 baud
		STA  DUART+CLOCK_B
		LDA #%01001100		;MRA0
		STA  DUART+MODE_A
		LDA #%01001000		;MRB0 - Bits 2 & 0 are also reserved in Ch B
		STA  DUART+MODE_B
		LDA #%01010011		;MRA1
		STA  DUART+MODE_A
		LDA #%11010011		;MRB1 - RTS handshaking
		STA  DUART+MODE_B
		LDA #%00000111		;MRA2
		STA  DUART+MODE_A
		LDA #%00010111		;MRB2 - CTS handshaking
		STA  DUART+MODE_B
		LDA #%00110000
		STA  DUART+ACR
		LDA #$05			;enable Tx & Rx
		STA  DUART+COMMAND_A
		STA  DUART+COMMAND_B
		RTS

DUART_INA:
;Disturbs: A - if byte recieved
;Parameters: None
;Returns: A - if byte received, it goes in here.
;		  C - set if byte received, clear if not
;Gets a byte from serial port A, if present.
		PHA
		LDA DUART+STATUS_A
		LSR		;Determine whether or not there is a character to receive.
		PLA
		BCC @out
		LDA DUART+FIFO_A
@out:
		RTS

DUART_OUTA:
;Disturbs: C - as return
;Parameters: A - Byte to send
;Returns: C - Clear if not sent, set if byte sent.
;Pushes a byte out serial port A, if it can be done. If it cannot be sent, then returns with carry clear.
		PHA					;get A out of the way, so we can use it.
		LDA DUART+STATUS_A
		LSR
		LSR
		LSR					;put TxRDY in the carry. It will provide the return value.
		PLA
		BCC @out
		STA DUART+FIFO_A	;send the byte
@out:
		RTS


;C wrappers for the assembly send and receive routines above.

;	unsigned char fastcall duart_a_available(void);
; Returns -1(true) if available, 0(false) otherwise.
_duart_a_available:
		PHP
		
		LDA DUART+STATUS_A
		LSR
		LSR
		LSR					;put TxRDY in the carry flag. The return value will be derived from this.
		
		LDA $FF				;assume available, set up to return true(-1)
		
		BCS @available
		LDA #$00			;if not available, return false(0)
		
@available:
		TAX
		PLP
		RTS


;	char fastcall duart_ina_nw(void);
; Returns whatever is in the DUART's channel A FIFO. Does not check that it is meaningful.
_duart_ina_nw:
		LDA DUART+FIFO_A
		LDX #$00			;Zero the high byte in case of type-promotion
		RTS

;	char fastcall duart_ina(void);
; Blocking version of _duart_ina_nw. Waits for a character on channel A, and returns that.
_duart_ina:
		PHP
		
@loop:	JSR DUART_INA
		BCC @loop
		
		LDX #$00			;Zero the high byte in case of type-promotion
		PLP
		RTS

;	unsigned char fastcall duart_outa_nw(char toSend);
; returns -1(true) if character sent, 0(false) if not.
_duart_outa_nw:
		PHP					;Save the processor status(I don't know if it's needed)
		
		JSR DUART_OUTA
		
		LDA #$FF			;set A to -1(assume it's sent).
		BCS @sent
		LDA #$00			;If it didn't send, set return to 0.
@sent:	TAX					;This function returns either 0 or -1. The high byte is the same as the low in both cases.
		PLP					;Restore the status register
		RTS

;	void fastcall duart_outa(char toSend);
; Blocking version of duart_outa_nw; sends byte and waits for completion.
_duart_outa:
		PHP
		
@loop:	JSR DUART_OUTA
		BCC @loop
		
		PLP
		RTS
