;interrupt.s - interrupt handler
; checks for a BRK, and returns from all valid IRQs

.import _stop
.export _irq_int, _nmi_int

.segment "CODE"

.PC02

;NMI service routine:
_nmi_int:	JSR _NMI_service
			RTI						;return from all NMIs

;IRQ service routine:
_irq_int:	PHX						; Save X register contents to stack
			TSX						; Transfer stack pointer to X
			PHA						; Save accumulator contents to stack
			INX						; Increment X so it points to the status
			INX						;   register value saved on the stack
			LDA $100,X				; Load status register contents
			AND #$10				; Isolate B status bit
			BNE break				; If B = 1, BRK detected

;hardware IRQ detected. hand over control to the ISR.
irq:		JSR _IRQ_service		; Requires a C function of the form "void IRQ_service(void)" or assembly routine _IRQ_service that consumes no arguments
			PLA						; Restore accumulator contents
			PLX						; Restore X register contents
			RTI						; Return from all IRQ interrupts

break:		JMP _stop				; If BRK is detected, something very bad
									;   has happened, so stop running
